<!DOCTYPE kpartgui>
<gui name="kdevcustommakemanager" library="libkdevcustommakemanager" version="4">
  <MenuBar>
  <Menu name="run">
   <text context="@title:menu">Run</text>
   <Action name="build_target"/>
  </Menu>
</MenuBar>
</gui>
